/**
 * Introduction
 *
 * @title ROV - Easy applications from C
 *
 * @copyright Jason Tribbeck / 7th ARM 1994-2001
 *
 * @authors Jason Tribbeck
 *
 * @file handextend.h
 *
 * @type subsystem
 *
 * @description
 *
 * The handextend subsystem is designed to add handlers for unknown WIMP messages. Each
 * handler can be claimed to be called for a particular WIMP message type and action.
 */

#ifndef __rov_handextend
#define __rov_handextend


#ifdef __cplusplus
extern "C"
{
#endif

/**
 * Definition of a handextend handler.
 * @param type the WIMP message type (17, 18 or 19)
 * @param block the WIMP message block.
 * @return non-zero indicating message has been dealt with.
 */
typedef int (*handproc)(int type,
			message_block *block);

/**
 * Initialise the handextend subsystem.
 * @note Do not claim the HANDLER_MESSAGE, the HANDLER_MESSAGEACK
 * nor the HANDLER_ACKNLOWEDGE in main.h if you want to use the
 * handextend subsystem.
 * @see main.h
 */
void handextend_init(void);

/**
 * Claim a handextend handler.
 * @param action the WIMP action to claim on.
 * @param type_flags defines which WIMP message types are applicable.
 * @param handler the handler function.
 * @note You can have many of these, as they are kept in a linked list.
 * @see WIMP message type flags
 */
void handextend_claim(int action,
		      int type_flags,
		      handproc handler);

/**
 * Release a handextend handler.
 * @param action the WIMP action to claim on.
 * @param type_flags defines which WIMP message types are applicable.
 * @param handler the handler function.
 * @note All of these must match exactly for it to be released.
 * @see WIMP message type flags
 */
void handextend_release(int action,
			int type_flags,
			handproc handler);

/**
 * @section WIMP message type flags
 * These are bitfields to generate the type_flags in handextend_claim()
 * and handextend_release()
 * @see handextend_claim()
 * @see handextend_release()
 */
#define HANDEXTEND_MESSAGE	(1<<0)
#define HANDEXTEND_MESSAGEACK	(1<<1)
#define HANDEXTEND_ACKNOWLEDGE	(1<<2)


#ifdef __cplusplus
}
#endif

#endif
