/**
 * Introduction
 *
 * @title ROV - Easy applications from C
 *
 * @copyright Jason Tribbeck / 7th ARM 1994-2001
 *
 * @authors Jason Tribbeck
 *
 * @file icons.h
 *
 * @type core
 *
 * @description
 *
 * The icon core system deals with all aspects of icons. It co-exists with the
 * canvas core system in that it extends handlers for canvas events that pertain
 * to icons.
 *
 * @note ROV icons claim memory for storing pointers to their handlers, unless
 * they are created as unhandled (called 'forgetful') icons. These forgetful icons
 * cannot have handlers attached to them, and are purely decorative.
 */

#ifndef __rov_icons
#define __rov_icons


#ifdef __cplusplus
extern "C"
{
#endif

#ifndef __rov_main
#include "main.h"
#endif

/**
 * Definition of an icon
 * @field text the text in the icon, or the name of the sprite.
 * @field indirecttext.text the pointer to the indirected text.
 * @field indirecttext.valid the pointer to the icon's validation text.
 * @field indirecttext.length the length of the pointed to text.
 */
typedef union {
  char text[12];
  struct {
    char *text;
    char *valid;
    int length;
  } indirecttext;
} icon_data;

/**
 * Initialise the icon core system
 */
void icon_init(void);

#ifdef OLD_ICONHEIGHTS
#define icon_createtext(aa,bb,cc,dd) icon_createoldtext(aa,bb,cc,dd)
#else
/**
 * Create a standard height text icon.
 * @param c the canvas.
 * @param x the (canvas) left hand coordinate.
 * @param y the (canvas) bottom coordinate.
 * @param text the pointer to the text icon.
 * @return the created icon.
 * @note This creates an icon of 52 OS units high.
 * @see icon_createoldtext()
 * @see icon_createrjtext()
 */
icon icon_createtext(canvas c,
		     int x,
		     int y,
		     char *text);
#endif

/**
 * Create a smaller height text icon.
 * @param c the canvas
 * @param x the (canvas) left hand coordinate.
 * @param y the (canvas) bottom coordinate.
 * @param text the pointer to the text icon.
 * @return the created icon.
 * @note This creates an icon of 44 OS units high.
 * @see icon_createtext()
 * @see icon_create()
 */
icon icon_createoldtext(canvas c,
			int x,
			int y,
			char *text);

/**
 * Create a right-aligned standard height text icon.
 * @param c the canvas.
 * @param x the (canvas) right hand coordinate.
 * @param y the (canvas) bottom coordinate.
 * @param text the pointer to the text.
 * @return the created icon.
 * @note This creates an icon of 52 OS units high.
 * @see icon_create()
 */
icon icon_createrjtext(canvas c,
		       int x,
		       int y,
		       char *text);

/**
 * Create a sprite icon.
 * @param c the canvas.
 * @param x the (canvas) left hand coordinate.
 * @param y the (canvas) bottom coordinate.
 * @param spr_area the sprite area (1 for the WIMP's area)
 * @param spr_name the name of the sprite.
 * @return the created icon.
 * @note The size of the icon is taken from the sprite itself. If the size
 * cannot be worked out (sprite non-existant, or mode invalid), then
 * undefined results occur.
 * @see icon_create()
 */
icon icon_createsprite(canvas c,
		       int x,
		       int y,
		       void *spr_area,
		       char *spr_name);

/**
 * Create an icon
 * @param c the canvas.
 * @param x0 the (canvas) left hand coordinate.
 * @param y0 the (canvas) bottom coordinate.
 * @param x1 the (canvas) right hand coordinate.
 * @param y1 the (canvas) top coordinate.
 * @param flags the icon flags.
 * @param data the icon data.
 * @return the created icon.
 * @see icon_data
 * @see Icon flags
 * @see icon_createforgetful()
 */
icon icon_create(canvas c,
		 int x0,
		 int y0,
		 int x1,
		 int y1,
		 int flags,
		 icon_data *data);

/**
 * Create a radio icon.
 * @param c the canvas.
 * @param x the (canvas) left hand coordinate.
 * @param y the (canvas) bottom coordinate.
 * @param text the text to follow after the radio icon.
 * @return the created icon.
 * @note This creates an icon of 52 OS units high.
 * @note Neither the button flags nor the ESG is set for this button to
 * work as expected.
 * @see icon_setgroup()
 * @see icon_setbuttontype()
 * @see icon_create()
 */
icon icon_createradio(canvas c,
		      int x,
		      int y,
		      char *text);

/**
 * Create an option button.
 * @param c the canvas.
 * @param x the (canvas) left hand coordinate.
 * @param y the (canvas) bottom coordinate.
 * @param text the text to follow after the option icon.
 * @return the created icon.
 * @note This creates an icon of 52 OS units high.
 * @note Neither the button flags nor the ESG is set for this button to
 * work as expected.
 * @see icon_setgroup()
 * @see icon_setbuttontype()
 * @see icon_create()
 */
icon icon_createoption(canvas c,
		       int x,
		       int y,
		       char *text);

/**
 * Create an information field (slabbed in).
 * @param c the canvas.
 * @param x the (canvas) left hand coordinate.
 * @param y the (canvas) bottom coordinate.
 * @param text the text in the information field.
 * @return the created icon.
 * @note This creates an icon of 52 OS units high, and uses the text
 * pointed to in order to get the width (using main_strwidth() from
 * main.h, and adding 24 to it).
 * @note If 2D has been configured, then the icon becomes a 2D white
 * background with black border icon.
 * @see icon_createfixedinfofield()
 * @see main.h
 * @see icon_create()
 */
icon icon_createinfofield(canvas c,
			  int x,
			  int y,
			  char *text);

/**
 * Create a fixed width information field (slabbed in).
 * @param c the canvas
 * @param x the (canvas) left hand coordinate.
 * @param y the (canvas) bottom coordinate.
 * @param text the text in the information field.
 * @param width the width of the icon.
 * @return the created icon.
 * @note This creates an icon of 52 OS units high.
 * @note If 2D has been configured, then the icon becomes a white
 * background with a black border.
 * @see icon_create()
 */
icon icon_createfixedinfofield(canvas c,
			       int x,
			       int y,
			       char *text,
			       int width);

/**
 * Create a button that is normally the default button (validation
 * string "r5,3").
 * @param c the canvas.
 * @param x the (canvas) left hand inside coordinate.
 * @param y the (canvas) bottom inside coordinate.
 * @param text the text in the button.
 * @param width the width of the inside of the button.
 * @return the created icon.
 * @note If 3D has been configured, the actual size of the button
 * is larger than the given sizes.
 * @note If 2D is set, then the icon becomes a beige background
 * with a black border.
 * @see icon_createbutton()
 * @see icon_create()
 */
icon icon_createdefaultbutton(canvas c,
			      int x,
			      int y,
			      char *text,
			      int width);

/**
 * Create a button.
 * @param c the canvas.
 * @param x the (canvas) left hand coordinate.
 * @param y the (canvas) bottom coordinate.
 * @param text the text in the button.
 * @param width the width of the button.
 * @return the created icon.
 * @note If 2D is set, then the icon becomes a white background
 * with a black border.
 * @see icon_createbutton()
 * @see icon_create()
 */
icon icon_createbutton(canvas c,
		       int x,
		       int y,
		       char *text,
		       int width);

/**
 * Create a menu button.
 * @param c the canvas.
 * @param x the (canvas) left hand coordinate.
 * @param y the (canvas) bottom coordinate.
 * @return the created icon.
 * @note If 2D is set, then the icon becomes a text icon with
 * a mid-grey background and a black border. 2D is a 44x44 OS
 * unit sized icon, moved up by 4 OS units (to vertically align
 * it with any information field next to it).
 * @note if 3D is set, then the icon becomes a sprite icon
 * called "gright" (with a depressed version called "pgright").
 * 3D is a 52x52 OS unit sized icon.
 * @see icon_create()
 */
icon icon_createmenubutton(canvas c,
			   int x,
			   int y);

/**
 * Create a writeable icon.
 * @param c the canvas.
 * @param x the (canvas) left hand coordinate.
 * @param y the (canvas) bottom coordinate.
 * @param str the pointer to the text in the icon.
 * @param valid the pointer to the validation string.
 * @param len the maximum length of the text in the icon.
 * @param width the width of the writeable icon.
 * @return the created icon.
 * @note There is no difference between the 2D and the 3D
 * versions.
 * @see icon_create()
 */
icon icon_createwriteable(canvas c,
			  int x,
			  int y,
			  char *str,
			  char *valid,
			  int len,
			  int width);

/**
 * Create a 'group box' icon.
 * @param c the canvas.
 * @param x0 the (canvas) left hand coordinate.
 * @param y0 the (canvas) bottom coordinate.
 * @param x1 the (canvas) right hand coordinate.
 * @param y1 the (canvas) top coordinate.
 * @param name the group name (at the top left of the box).
 * @note This creates two forgetful icons.
 * @see icon_create()
 * @see icon_createforgetful()
 */
void icon_createbox(canvas c,
		    int x0,
		    int y0,
		    int x1,
		    int y1,
		    char *name);

/**
 * Remove an icon from the screen.
 * @param ic the icon to remove.
 * @note Do not call this for removing forgetful icons.
 * @see icon_removeforgetful()
 */
void icon_remove(icon ic);

/**
 * Get the WIMP icon handle.
 * @param ic the icon.
 * @return the WIMP icon handle.
 */
int icon_handle(icon ic);

/**
 * Find an icon
 * @param c the canvas to search in.
 * @param i the WIMP icon handle to search for.
 * @return the icon, or NULL if not found.
 */
icon icon_find(canvas c,
	       int i);

/**
 * Get the canvas.
 * @param ic the icon.
 * @return the canvas in which the icon presides.
 */
canvas icon_canvas(icon ic);

/**
 * Create an icon from an existing WIMP icon.
 * @param c the canvas
 * @param i the WIMP icon handle.
 * @return the generated icon.
 */
icon icon_useexisting(canvas c,
		      int i);

#ifdef OLDSTYLE
void icon_addhandler(icon, int, void *); 			/* Attach a click handler		 */
#else
/**
 * Add a handler to an icon.
 * @param ic the icon.
 * @param event the icon event.
 * @param ... the pointer to the event handler.
 * @see icon_clickhandler()
 * @see icon_menuhandler()
 * @see icon_keyhandler()
 * @see icon_carethandler()
 * @see icon_helphandler()
 */
void icon_addhandler(icon ic,
		     int event,
		     ...);
#endif

/**
 * Set the foreground colour of the icon.
 * @param ic the icon.
 * @param col the WIMP colour.
 */
void icon_setfore(icon ic,
		  int col);

/**
 * Set the background colour of an icon.
 * @param ic the icon.
 * @param col the WIMP colour.
 */
void icon_setback(icon ic,
		  int col);

/**
 * Set the font handle for an icon.
 * @param ic the icon.
 * @param handle the font handle.
 * @see font.h
 */
void icon_setfont(icon ic,
		  int handle);

/**
 * Redraw an icon.
 * @param ic the icon.
 * @note This calls Wimp_SetIconState with no effects.
 */
void icon_update(icon ic);

/**
 * Set/reset the filled icon flag.
 * @param ic the icon.
 * @param filled non-zero for filled, zero for unfilled.
 */
void icon_fill(icon ic,
	       int filled);

/**
 * Set/reset the border icon flag.
 * @param ic the icon.
 * @param border non-zero for having a border, zero for unbordered.
 */
void icon_border(icon ic,
		 int border);

/**
 * Select/deselect an icon.
 * @param ic the icon.
 * @param selected non-zero for selected, zero for selected.
 */
void icon_select(icon ic,
		 int selected);

/**
 * Fade/unfade an icon.
 * @param ic the icon.
 * @param faded non-zero for faded, zero for faded.
 */
void icon_fade(icon ic,
	       int faded);

/**
 * Find out if an icon is selected.
 * @param ic the icon.
 * @return non-zero for selected.
 */
int icon_selected(icon ic);

/**
 * Find out if an icon is faded.
 * @param ic the icon.
 * @return non-zero for faded.
 */
int icon_faded(icon ic);

/**
 * Set the button type of an icon.
 * @param ic the icon.
 * @param type the button type.
 * @see Icon button types.
 */
void icon_setbuttontype(icon ic,
			int type);

/**
 * Set the ESG of an icon.
 * @param ic the icon.
 * @param group the group number (0-31).
 */
void icon_setgroup(icon ic,
		   int group);

/**
 * Set the horizontal justification of an icon.
 * @param ic the icon.
 * @param justify the justification.
 * @see Icon justification.
 */
void icon_setjustification(icon ic,
			   int justify);

/**
 * Create a 'forgetful' icon.
 * @param c the canvas.
 * @param x0 the (canvas) left hand coordinate.
 * @param y0 the (canvas) bottom coordinate.
 * @param x1 the (canvas) right hand coordinate.
 * @param y1 the (canvas) top coordinate.
 * @param flags the icon flags.
 * @param data the icon data.
 * @return the WIMP icon handle.
 * @see icon_data
 * @see Icon flags
 * @see icon_create()
 */
int icon_createforgetful(canvas c,
			 int x0,
			 int y0,
			 int x1,
			 int y1,
			 int flags,
			 icon_data *data);

/**
 * Remove a 'forgetful' icon.
 * @param c the canvas.
 * @param i the WIMP icon handle.
 */
int icon_removeforgetful(canvas c,
			 int i);

/**
 * Get the address of the text data in an icon.
 * @param ic the icon.
 * @return the text pointer.
 */
char *icon_textpointer(icon ic);

/**
 * Get the address of the validation string in an icon.
 * @param ic the icon.
 * @return the validation pointer.
 */
char *icon_validpointer(icon ic);

/**
 * Set the caret position in an icon.
 * @param ic the icon.
 * @param x the x coordinate.
 * @param y the y coordinate.
 * @param index the index in the string.
 */
void icon_setcaret(icon ic,
		   int x,
		   int y,
		   int index);

/**
 * Validation string for the radio icon - 'Sradiooff,radioon'.
 */
extern char radio_valid[];

/**
 * Validation string for the option icon - 'Soptoff,opton'.
 */
extern char option_valid[];

/**
 * The null icon.
 */
#define ICON_NULL	((icon)0)

/**
 * @section Icon button types
 */
#define ICON_BNONE		(0)
#define ICON_BALWAYS		(1)
#define ICON_BREPEAT		(2)
#define ICON_BCLICK		(3)
#define ICON_BRELEASE		(4)
#define ICON_BDOUBLE		(5)
#define ICON_BCLICKDRAG		(6)
#define ICON_BDOUBLEDRAG	(7)
#define ICON_BRELEASEDRAG	(8)
#define ICON_BMENUTYPE		(9)
#define ICON_BCLICKDOUBLEDRAG	(10)
#define ICON_BSELECTDRAG	(11)
#define ICON_BWRITEDRAG		(14)
#define ICON_BWRITEABLE		(15)

/**
 * @section Icon flags
 */
#define ICON_TEXT               (1<<0)
#define ICON_SPRITE		(1<<1)
#define ICON_BORDER		(1<<2)
#define ICON_HCENTRED		(1<<3)
#define ICON_VCENTRED		(1<<4)
#define ICON_FILLED		(1<<5)
#define ICON_ANTIALIASED	(1<<6)
#define ICON_NEEDSREDRAWHELP	(1<<7)
#define ICON_INDIRECTED		(1<<8)
#define ICON_RIGHTJUSTIFIED	(1<<9)
#define ICON_BUTTONSHIFT	(12)
#define ICON_BUTTONFLAGS	(15<<12)
#define ICON_ESGSHIFT		(16)
#define ICON_ESGFLAGS		(31<<16)
#define ICON_SELECTED		(1<<21)
#define ICON_FADED		(1<<22)
#define ICON_DELETED		(1<<23)
#define ICON_FORECOLOURSHIFT	(24)
#define ICON_FORECOLOUR		(15<<24)
#define ICON_BACKCOLOURSHIF	(28)
#define ICON_BACKCOLOUR		(15<<28)

/**
 * @section Icon event codes
 */
#define ICON_CLICKHANDLER	(0)
#define ICON_MENUHANDLER	(1)
#define ICON_KEYHANDLER		(2)
#define ICON_CARETHANDLER	(3)
#define ICON_HELPHANDLER	(4)

/**
 * @section Icon justification
 */
#define ICON_TSL_TBST           (0x000000000)
#define ICON_TSR_TBST		(0x100000000)
#define ICON_SLT12R_TSC		(0x000001000)
#define ICON_TLSR_TSC		(0x100001000)
#define ICON_TSC_TBST		(0x000000100)
#define ICON_TSC_TTSB		(0x000001100)
#define ICON_TSC_TSC		(0x100000100)
#define ICON_TRSL_TSC		(0x100001100)

/**
 * Icon click handler function prototype.
 * @param c the canvas.
 * @param ic the icon.
 * @param mouse_str the mouse event block.
 */
typedef void (*icon_clickhandler)(canvas c,
				  icon ic,
				  canvas_mousestr *mouse_str);

/**
 * Icon menu handler function prototype.
 * @param c the canvas.
 * @param ic the icon.
 * @param mouse_str the mouse event block.
 */
typedef void (*icon_menuhandler)(canvas c,
				 icon ic,
				 canvas_mousestr *mouse_str);

/**
 * Icon key handler function prototype.
 * @param c the canvas.
 * @param ic the icon.
 * @param key_str the key event block.
 */
typedef void (*icon_keyhandler)(canvas c,
				icon ic,
				canvas_keystr *key_str);

/**
 * Icon caret handler function prototype.
 * @param c the canvas.
 * @param ic the icon.
 * @param caret_str the caret event block.
 */
typedef void (*icon_carethandler)(canvas c,
				  icon ic,
				  canvas_caretstr *caret_str);

/**
 * Icon help handler function prototype.
 * @param c the canvas.
 * @param ic the icon.
 */
typedef void (*icon_helphandler)(canvas c,
				 icon ic);


#ifdef __cplusplus
}
#endif

#endif
