/**
 * Introduction
 *
 * @title ROV - Easy applications from C
 *
 * @copyright Jason Tribbeck / 7th ARM 1994-2001
 *
 * @authors Jason Tribbeck
 *
 * @file slider.h
 *
 * @type helper
 *
 * @description
 *
 * The slider helper generates standard sliders on canvasses.
 *
 * See the slider tutorial for more information.
 *
 * @note Sliders are internally held as integers, which have a number
 * of decimal places afterwards. This means that the integer value of
 * 31412 with 4 decimal places equates to 3.1412, and with 2 decimal
 * places 314.12.
 *
 * @note Canvasses must have redraw handlers that call slider_redraw()
 * in order to display the sliders.
 * @see slider_redraw()
 */

#ifndef __rov_slider
#define __rov_slider


#ifdef __cplusplus
extern "C"
{
#endif

/**
 * The definition of a slider.
 */
typedef void *slider;

/**
 * The definition of a slider event handler.
 * @param s the slider.
 * @param event the event code.
 * @param val the new value.
 * @return non-zero for handled, zero for not handled.
 * @see Slider event codes
 */
typedef int (*slider_changeval)(slider s,
				int event,
				int val);

/**
 * Create a slider.
 * @param c the canvas to create the canvas on.
 * @param x the (canvas) left hand coordinate.
 * @param y the (canvas) bottom coordinate.
 * @param l the slider length.
 * @param flags the flags for the slider.
 * @param comment the text for the comment (if bit 0 of flags is set).
 * @param units the text for the units (if bit 2 of flags is set).
 * @param value the default value.
 * @param min the minimum value.
 * @param max the maximum value.
 * @param step the step between valid values.
 * @param decimals the number of decimal places.
 * @param valuewidth the width of the value text box.
 * @param unitwidth the width of the unit icon.
 * @param handler the slider event handler.
 * @see Slider flags
 * @note the comment appears to the left of the left hand coordinate.
 * @note the slider subsystem will claim the mouse click events for
 * this canvas.
 */
slider slider_create(canvas c,
		     int x,
		     int y,
		     int l,
		     int flags,
		     char *comment,
		     char *units,
		     int value,
		     int min,
		     int max,
		     int step,
		     int decimals,
		     int valuewidth,
		     int unitwidth,
		     slider_changeval handler);

/**
 * Slider redrawing
 * @param c the canvas being redrawn.
 * @param x0 the (canvas) left hand coordinate.
 * @param y0 the (canvas) bottom coordiate.
 * @param x1 the (canvas) right hand coordinate.
 * @param y1 the (canvas) top coordinate.
 * @note This is normally called from the canvas redraw event handler.
 */
void slider_redraw(canvas c,
		   int x0,
		   int y0,
		   int x1,
		   int y1);

/**
 * Set the value of a slider.
 * @param s the slider.
 * @param value the value.
 */
void slider_setvalue(slider s,
		     int value);

/**
 * Set the colour of a slider.
 * @param s the slider.
 * @param col the WIMP colour.
 */
void slider_setcolour(slider s,
		      int col);

/**
 * @section Slider flags
 */
#define SLIDER_COMMENT (1<<0)
#define SLIDER_TEXT    (1<<1)
#define SLIDER_UNITS   (1<<2)
#define SLIDER_ARROWS  (1<<3)
#define SLIDER_CENTRE  (1<<4)
#define SLIDER_ALL     (31)

/**
 * @section Slider event codes
 */
#define SLIDER_RETURNPRESSED (0)
#define SLIDER_INCREMENT     (1)
#define SLIDER_DECREMENT     (2)
#define SLIDER_DRAGTO        (3)
#define SLIDER_DRAGEND       (4)
#define SLIDER_KEYPRESSED    (5)


#ifdef __cplusplus
}
#endif

#endif
